/*
    space.h - root cell structure and real-to-integer coordinate mappings
    
    Copyright (C) 2009  Laurent Poirrier
    This file is part of the lin-kd-tree library.

    The lin-kd-tree library is free software: you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation, version 3 of the License.

    The lin-kd-tree library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __KDTREE_SPACE_H
#define __KDTREE_SPACE_H
#include "kd-tree.h"

/*
  The kd-tree space functions handle the real-to-integer coordinate
  mappings.
*/


// *********************************************************************
// kd-tree space
// *********************************************************************
#define KDTREE_SCALAR		double

typedef struct kdtree_sphere_t {
	KDTREE_SCALAR x, y, z, r;
} kdtree_sphere;

typedef struct kdtree_aabb_t {
	KDTREE_SCALAR x0, x1, y0, y1, z0, z1;
} kdtree_aabb;

typedef struct kdtree_space_t {
	KDTREE_SCALAR x0, y0, z0;
	KDTREE_SCALAR x_width, y_width, z_width;
	KDTREE_SCALAR x_mult, y_mult, z_mult;
} kdtree_space;


// *********************************************************************
// Functions
// *********************************************************************

void kdtree_set_space(kdtree_space *s,
	KDTREE_SCALAR x0, KDTREE_SCALAR x1, KDTREE_SCALAR y0, KDTREE_SCALAR y1,
	KDTREE_SCALAR z0, KDTREE_SCALAR z1);
void kdtree_set_isospace(kdtree_space *s,
	KDTREE_SCALAR x0, KDTREE_SCALAR x1, KDTREE_SCALAR y0, KDTREE_SCALAR y1,
	KDTREE_SCALAR z0, KDTREE_SCALAR z1);

int kdtree_pack_aabb(kdtree_space *s, kdtree_iaabb *dst, kdtree_aabb *src);
int kdtree_pack_sphere(kdtree_space *s, kdtree_iaabb *dst, kdtree_sphere *src);

// debug
void kdtree_retreive_aabb(kdtree_space *s, kdtree_iaabb *bb, kdtree_aabb *dst);
void kdtree_node_aabb(kdtree_space *s, kdtree_node *k, kdtree_aabb *dst);


#endif



